#include <time.h>
#include "gpsd.h"
#include "gpsparser.h"
#include <math.h>

extern DBPOINT *waypoint;
extern int waypoint_count;

char *NavModes[] =
{
    "Init req'd", "Initialized", "NAV 3D", "3", "4", "5", "Dead Rec", "7",
    "BaseStation", "CodeNav", "RTK"
};

unsigned char hour;
unsigned char minute;
double seconds;
unsigned char day;
unsigned char month;
unsigned short year;
double latitude, latitude_deg;
double longitude, longitude_deg;
float altitude;
float clockbias;
float clockdrift;
float velocitynorth;
float velocityeast;
float velocityup;
float hfom;
float vfom;
float gpstime;
unsigned char navmode;
unsigned char numofsv;
unsigned char reserved;
unsigned short weekno;

char nmeabuffer[256];

void rxparser20(struct rawmsgformat msg)
{
    int lat, lon;
    char *buffer = nmeabuffer;
    DBPOINT *p = &waypoint[waypoint_count];
    struct tm tm;

    hour = msg.message[0];
    minute = msg.message[1];
    raw2ARMdouble(&msg.message[2], &seconds);
    day = msg.message[10];
    month = msg.message[11];
    raw2short(&msg.message[12], &year);
    raw2ARMdouble(&msg.message[14], &latitude);
    raw2ARMdouble(&msg.message[22], &longitude);
    raw2ARMfloat(&msg.message[30], &altitude);
//    raw2ARMfloat(&msg.message[34], &clockbias);
//    raw2ARMfloat(&msg.message[38], &clockdrift);
    raw2ARMfloat(&msg.message[42], &velocitynorth);
    raw2ARMfloat(&msg.message[46], &velocityeast);
    raw2ARMfloat(&msg.message[50], &velocityup);
//    raw2ARMfloat(&msg.message[54], &hfom);
//    raw2ARMfloat(&msg.message[58], &vfom);
//    raw2ARMfloat(&msg.message[62], &gpstime);
    navmode = msg.message[66];
    numofsv = msg.message[67];
//    reserved = msg.message[68];
//    raw2short(&msg.message[69], &weekno);

    /* Convert to degrees */
    latitude_deg = latitude * 180 / M_PI;
    longitude_deg = -longitude * 180 / M_PI;
    
    /* Spew data point out to NMEA device */
    lat = (int) latitude_deg;
    lon = (int) longitude_deg;
#ifdef GPRMC
    sprintf(buffer, "$GPRMC,%02d%02d%02d,A,%02d%09.6f,N,%03d%09.6f,W,0,0,"
    	"%02d%02d%02d,0,W,A\r\n", hour, minute, (int)seconds,
    	lat, (latitude_deg - lat) * 60,
    	lon, (longitude_deg - lon) * 60,
    	day, month, year % 100);
    write(nmeafd, buffer, strlen(buffer));
#endif    	
//#ifdef GPGGA
    sprintf(buffer, "$GPGGA,%02d%02d%02d.00,%02d%09.6f,N,%03d%09.6f,W,1,%d,"
    	"01.0,%.2f,M,-33.0,M,,\r\n", hour, minute, (int)seconds,
    	lat, (latitude_deg - lat) * 60,
    	lon, (longitude_deg - lon) * 60,
    	numofsv, altitude);
    write(nmeafd, buffer, strlen(buffer));
//#endif    	

    /* Don't log points if velocity is < 3km/hr */
    if ((velocitynorth < 1.0) && (velocityeast < 1.0)) {
//    	return;
    }

    /* Popualate the DBPOINT structure */
    tm.tm_sec = (int)seconds;
    tm.tm_min = minute;
    tm.tm_hour = hour;
    tm.tm_mday = day;
    tm.tm_mon = month - 1;
    tm.tm_year = year - 1900;

    p->timestamp = mktime(&tm);
    p->latitude = (float)latitude;
    p->longitude = (float)longitude;
    p->altitude = (short)altitude;
    p->mode_sv = ((navmode & 0xf) << 4) | (numofsv & 0xf);

    if (waypoint_count < MAX_WAYPOINTS-1)
    	waypoint_count++;

//#if 0
    printf("%02d/%02d/%02d %02d:%02d:%02d (%f %f %d) (%f, %f) %02x\n", 
	tm.tm_year+1900, tm.tm_mon, tm.tm_mday,
	tm.tm_hour, tm.tm_min, tm.tm_sec,
//	p->latitude, p->longitude, p->altitude,
	latitude_deg, longitude_deg, p->altitude,
	velocitynorth, velocityeast,
	p->mode_sv);
//#endif
}

/* Raw msg data dump */
void rxparser0(struct rawmsgformat msg)
{
#if 0
    int count;
    fprintf(stderr, "%02x\n", msg.start);
    fprintf(stderr, "%02x\n", msg.msgid);
    fprintf(stderr, "%02x\n", msg.msgidcmpl);
    fprintf(stderr, "%02x\n", msg.msglen);
    for (count = 0; count < msg.msglen; count++) {
	fprintf(stderr, "%02x ", msg.message[count]);
    }
    fprintf(stderr, "\n%02x\n", msg.checksumlsb);
    fprintf(stderr, "%02x\n\n", msg.checksummsb);
#endif
    return;
}

void rxparser6(struct rawmsgformat msg)
{
#if 0
    unsigned int datasetnum;
    unsigned int svnumtype;
    unsigned int carrierfreq;
    float snr;
    unsigned int reserved1;
    unsigned int status;
    unsigned int foo;
    int i;

    printw("%u\n", msg.msgid);

    for (i = 0; i < 6; i++) {
        foo = i * 14;
	datasetnum = msg.message[0 + foo];
	svnumtype = msg.message[1 + foo];
	raw2int(&msg.message[2 + foo], &carrierfreq);
	raw2ARMfloat(&msg.message[6 + foo], &snr);
	raw2int(&msg.message[10 + foo], &reserved1);
	status = msg.message[14 + foo];

	mvprintw(15+i,1,"Chan %2d: SNR=%5.2f Status=%04x", 
		i, snr, status);
    }
#endif
}

void rxparser7(struct rawmsgformat msg)
{
#if 0
    unsigned int datasetnum;
    unsigned int svnumtype;
    unsigned int carrierfreq;
    float snr;
    unsigned int reserved1;
    unsigned int status;
    unsigned int foo;
    int i;

    printw("%u\n", msg.msgid);

    for (i = 0; i < 6; i++) {
        foo = i * 14;
	datasetnum = msg.message[0 + foo];
	svnumtype = msg.message[1 + foo];
	raw2int(&msg.message[2 + foo], &carrierfreq);
	raw2ARMfloat(&msg.message[6 + foo], &snr);
	raw2int(&msg.message[10 + foo], &reserved1);
	status = msg.message[14 + foo];

	mvprintw(15+i,40,"Chan %2d: SNR=%5.2f Status=%04x", 
		i + 6, snr, status);
    }
#endif
}

float cmc2float(char *x)
{
    float val;
    raw2ARMfloat(x, &val);
    return val;
}

double cmc2double(char *x)
{
    double val;
    raw2ARMdouble(x, &val);
    return val;
}

void rxparser47(struct rawmsgformat msg)
{
#if 0
    int base_status = msg.message[0] & 0x03;
    int baud_rate = msg.message[0] >> 2 & 0x07;
    double survey_time_remaining = cmc2double(&msg.message[6 - 5]);
    float base_station_cep = cmc2float(&msg.message[14 - 5]);
    double base_station_lat = cmc2double(&msg.message[18 - 5]);
    double base_station_long = cmc2double(&msg.message[26 - 5]);
    double base_station_height = cmc2double(&msg.message[34 - 5]);

    printw("Status %d\nBaud Rate 5d\n", base_status, baud_rate);
    printw("Time Remaining %.10f\n", survey_time_remaining);
    printw("%.10f\n", base_station_cep);
    printw("%.10f\n", base_station_lat);
    printw("%.10f\n", base_station_long);
    printw("%.10f\n", base_station_height);
    printw("\n");
    refresh();
#endif
}

void rxparsers33 (struct rawmsgformat msg)
{
#if 0
    int number = msg.message[5-5] & 0xf;
    int i;
    for (i=0; i<number; i++) {
      int sv_num = msg.message[6-5] & 0xf;
      int sv_stat = (msg.message[6-5] >> 4) & 0x7;
      int diff = (msg.message[6-5] >> 7) & 0x1;
    }
#endif
}
