
/*
 * This is machine specific.
 * The ARM IEEE754 double-precision format
 * has a mangled mantissa while the
 * single-precision format is "normal", requiring
 * only a 1-1 bit-mapping to convert.
 * see the ARM and Intel /usr/include/ieee754.h for details
 * foo is the address of the first element of the
 * double in the raw message.
 */
inline void raw2ARMdouble(unsigned char *foo, double *bar)
{
    union {
	double d;
	unsigned char c[8];
    } glorp;

    glorp.c[4] = *foo;
    glorp.c[5] = *(foo + 1);
    glorp.c[6] = *(foo + 2);
    glorp.c[7] = *(foo + 3);
    glorp.c[0] = *(foo + 4);
    glorp.c[1] = *(foo + 5);
    glorp.c[2] = *(foo + 6);
    glorp.c[3] = *(foo + 7);

    *bar = glorp.d;
    
//    *bar = *((double *)foo);
}

/*
 * Converse of above
 * foo is pointer to first element of array to
 * hold the 8 char raw version of the double
 * converted back to standard IEEE format in the
 * process.
 */
inline void ARMdouble2raw(unsigned char *foo, double *bar)
{
    union {
	double d;
	unsigned char c[8];
    } glorp;

    glorp.d = *bar;
    *foo = glorp.c[4];
    *(foo + 1) = glorp.c[5];
    *(foo + 2) = glorp.c[6];
    *(foo + 3) = glorp.c[7];
    *(foo + 4) = glorp.c[0];
    *(foo + 5) = glorp.c[1];
    *(foo + 6) = glorp.c[2];
    *(foo + 7) = glorp.c[3];
    
//    *((double *)foo) = *bar;
}


inline void raw2ARMfloat(unsigned char *foo, float *bar)
{
    union {
	float f;
	unsigned char c[4];
    } glorp;

    glorp.c[0] = *foo;
    glorp.c[1] = *(foo + 1);
    glorp.c[2] = *(foo + 2);
    glorp.c[3] = *(foo + 3);

    *bar = glorp.f;
}


inline void ARMfloat2raw(unsigned char *foo, float *bar)
{
    union {
	float f;
	unsigned char c[4];
    } glorp;

    glorp.f = *bar;
    *foo = glorp.c[0];
    *(foo + 1) = glorp.c[1];
    *(foo + 2) = glorp.c[2];
    *(foo + 3) = glorp.c[3];
}

inline void raw2int(unsigned char *foo, int *bar)
{
    union {
	int i;
	unsigned char c[4];
    } glorp;

    glorp.c[0] = *foo;
    glorp.c[1] = *(foo + 1);
    glorp.c[2] = *(foo + 2);
    glorp.c[3] = *(foo + 3);

    *bar = glorp.i;
}

inline void int2raw(unsigned char *foo, int *bar)
{
    union {
	int i;
	unsigned char c[4];
    } glorp;

    glorp.i = *bar;
    *foo = glorp.c[0];
    *(foo + 1) = glorp.c[1];
    *(foo + 2) = glorp.c[2];
    *(foo + 3) = glorp.c[3];
}

inline void raw2short(unsigned char *foo, short *bar)
{
    union {
	short s;
	unsigned char c[2];
    } glorp;

    glorp.c[0] = *foo;
    glorp.c[1] = *(foo + 1);

    *bar = glorp.s;
}

inline void short2raw(unsigned char *foo, short *bar)
{
    union {
	short s;
	unsigned char c[2];
    } glorp;

    glorp.s = *bar;
    *foo = glorp.c[0];
    *(foo + 1) = glorp.c[1];
}
